@@ -5,6 +5,7 @@ from .cpp_handler import CppHandler
 from .go_handler import GoHandler
 from .javascript_handler import JavaScriptHandler
 from .python_handler import PythonHandler
+from .r_handler import RHandler
 from .ruby_handler import RubyHandler
 
 __all__ = [
@@ -16,5 +17,6 @@ __all__ = [
     "PlotDetectionConfig",
     "PlotLibrary",
     "PythonHandler",
+    "RHandler",
     "RubyHandler",
 ]
